;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x2135464ef52708ac9e02109f6eacda8f . "./code.cg2js/cg-acache.fasl")
(#xd4510f64c265ee0073be0895c1d4f783 . "./code.cg2js/cg-base.fasl")
(#x31927b809c9571b5b6f73d2bbe829388 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xad78bd2f6804797437a05e7e827f7d76 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x54dd837511a7b2f88ae24dee6ef22fa5 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x3fa22b7f135f3a53b7fb84e69f670b96 . "./code.cg2js/cg-button.fasl")
(#x5157cb1aa100dc6302aaaad82ed1edb7 . "./code.cg2js/cg-calendar.fasl")
(#x41b8bbece2da0ba9bc16f0f21eebbbb7 . "./code.cg2js/cg-caret.fasl")
(#x44d4f174fa3fefbdad7c0db85dd74043 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xb554e0224d2192febb8f134fc7f000d8 . "./code.cg2js/cg-chart-widget.fasl")
(#x878162dceb48975e4cd2e0bb1bdfdd13 . "./code.cg2js/cg-check-box.fasl")
(#x49776cdd60cc2e9a9fe65931db60ffe4 . "./code.cg2js/cg-choice-list.fasl")
(#xdc59f6b65f18b8ab3e25955d73e46f70 . "./code.cg2js/cg-choose-printer.fasl")
(#x0318b1d02b16978c40b8810dc11b78c7 . "./code.cg2js/cg-class-grid.fasl")
(#xca74680a1aa1cdf8569a8a5376a4d549 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x77f47a077016d587c221cc2599446d1e . "./code.cg2js/cg-class-support.fasl")
(#xb16be4d276bc5d1e620215583b2dc29a . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xd6a8ea86b653aaa444778779c74d6871 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xa83f209d3bba05fed76ab33e70e6c597 . "./code.cg2js/cg-clipboard.fasl")
(#xedcae2b77200b1239f2253099f30b722 . "./code.cg2js/cg-color-dialog.fasl")
(#x90ec0fcec62bcfa62c1f7f4a7e0c182c . "./code.cg2js/cg-combo-box.fasl")
(#xc40aef0f6a1aa67b63dd2995bf7e851c . "./code.cg2js/cg-common-control.fasl")
(#x3d3fce763dffd123175ab1d76bc3819a . "./code.cg2js/cg-comtab.fasl")
(#xa7437f4b960a31235c77c8607d0b7177 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x3e7598e92dc8c78460471baeb56701ef . "./code.cg2js/cg-curve.fasl")
(#x30d3be0a1fe0a6da4442598fbd586209 . "./code.cg2js/cg-dde-utils.fasl")
(#xd909ab1c10c69a63d105411f4aec301c . "./code.cg2js/cg-dev.fasl")
(#xe33ab5aea0d4bcc8c1d76cac70a61a13 . "./code.cg2js/cg-dialog-item.fasl")
(#x6eef0cd21a08b7336b708fa7e2ce7fcb . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xe246d42ca9796013327ff7446081f7cb . "./code.cg2js/cg-directory-dialog.fasl")
(#x4194f8fab8e870cac2ae88b096373f56 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xf46f2d1f3b1d53bf9aaaa384f85b9f5a . "./code.cg2js/cg-drag-and-drop.fasl")
(#x2a0dd095b0bdcf243d64bbf23c3f5ba3 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x24a848b6ae6ec0a7f84398d4ece724b6 . "./code.cg2js/cg-drawable.fasl")
(#x4f21f1e39682a1687640874f7715f3ee . "./code.cg2js/cg-dropping-outline.fasl")
(#x4f1f22fc6dd6f1d6f4bed48a5df16d6d . "./code.cg2js/cg-edit-in-place.fasl")
(#xb5f5ab2d6e3959128aff3540afef9152 . "./code.cg2js/cg-editable-text.fasl")
(#xd3dc16134918b564efe934bcaba89420 . "./code.cg2js/cg-file-dialog.fasl")
(#x07348909f9f7df1a8ef164aa1730f6ff . "./code.cg2js/cg-fill-texture.fasl")
(#x07193c0fedec3898903b095b3c382d43 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x8f091df6ec3efcf0768cac5fb38a75ac . "./code.cg2js/cg-font-dialog.fasl")
(#x51b1a85b10db02766315311d9a0714b9 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xec17dae8efb021b6d8e30024bd83cdd9 . "./code.cg2js/cg-get-pixmap.fasl")
(#x4a226bafaed0abac94148d81fcbd06b6 . "./code.cg2js/cg-get-position.fasl")
(#x764eff1bce94a3b8f4ec79450a1c540e . "./code.cg2js/cg-graphics-context.fasl")
(#xa110438c3ceb1216d2a2eb3aff68d4cd . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x850d4213304ce62307e6cea0845f1fcf . "./code.cg2js/cg-grid-widget.fasl")
(#x5818bc7234e996ccf1a9b1b84e7e1ad6 . "./code.cg2js/cg-group-box.fasl")
(#xa61880547f28e04ece4fc2849642c77a . "./code.cg2js/cg-header-control.fasl")
(#xdb8f63b82310fd9fbff225037e36fb93 . "./code.cg2js/cg-hotspot.fasl")
(#x5c671d0462439cc6ab3d633ebc4ad42b . "./code.cg2js/cg-html-dialog.fasl")
(#x194c0357a11a868d6bf6cbaa9664a4a6 . "./code.cg2js/cg-html-widget.fasl")
(#x031772e8889ad66b6a2ea091d56d29bd . "./code.cg2js/cg-icon-pixmap.fasl")
(#x8ac341b7e668e29fcec9bd038c8d03d3 . "./code.cg2js/cg-icon.fasl")
(#x4103d1508bd2323628f935fb3aeefc8e . "./code.cg2js/cg-ie.fasl")
(#xefea6d56a8e4ebec17241279ec9d04bc . "./code.cg2js/cg-intersect-posbox.fasl")
(#x9935c4e7cf1fc0f706353b523d99288b . "./code.cg2js/cg-intersect.fasl")
(#xc0069d9afb6958ee31ce727168dc3292 . "./code.cg2js/cg-item-list.fasl")
(#xc428b0d8afb495ca6ffd636492e44d40 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x565310b3d581450f53b8fea908d1a507 . "./code.cg2js/cg-lamp.fasl")
(#xf07d1562955001220ad7386a618b8859 . "./code.cg2js/cg-layout.fasl")
(#x16944178929dfcf1931ea647c1becab3 . "./code.cg2js/cg-lettered-menu.fasl")
(#x17cf49e992ee8073ca1b48d55bde8156 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xede5669dd98c131c69491b8ba1e0c5d9 . "./code.cg2js/cg-lisp-text.fasl")
(#xb689cfa542faf757d0949240c4eff490 . "./code.cg2js/cg-lisp-widget.fasl")
(#xcd565aade1c96708a9e8d6ea2f665c48 . "./code.cg2js/cg-list-view.fasl")
(#x705c697edf248e9d61047a16c937c27b . "./code.cg2js/cg-mci.fasl")
(#xf911d4e50695adfdd6a3acbfa25c463f . "./code.cg2js/cg-menu-tooltip.fasl")
(#x090d1efbb85a6f2d5e688b8e923c0b17 . "./code.cg2js/cg-menu.fasl")
(#x209d64015d82b56ad75a0347d1e1876b . "./code.cg2js/cg-message-dialog.fasl")
(#x3ad734cef073f9e78e39698f6544e7f9 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xfa5f6dd940d682b7faadc965d0adedd4 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xafbed27b7fab824f941449908cf04f5f . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x8fb60a7317f83ae10ec4cdf9304f3157 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x3e3e28c803c02c75236b98e8eee4a073 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xd84371ef6c2cdcfcc38e5d5e74e0ed93 . "./code.cg2js/cg-nodes.fasl")
(#xb76ef60029c039bb3214cfdc2c06a335 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x95f69835281cc1e6503f2d8277f68337 . "./code.cg2js/cg-object-editor.fasl")
(#x90f5dedbb6627797145918320ac3b9d1 . "./code.cg2js/cg-ocx.fasl")
(#x5544d24acc4f2d36c364cd2eaa13d77b . "./code.cg2js/cg-os-widget.fasl")
(#xbb8c8028245e12cbb0d23600a5be6e62 . "./code.cg2js/cg-os-window.fasl")
(#xb3ce9f3f635e1636d93b3507c1d6e079 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x8f7a626da67db82aa2fdbe14844899c8 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x18572082bb3126c3875f38b3af229e04 . "./code.cg2js/cg-outline.fasl")
(#xf6973d3678ea4f7efd7706b2f2767ff2 . "./code.cg2js/cg-palette.fasl")
(#xad348f9b6a83b0cc7c051b2bacca4665 . "./code.cg2js/cg-paren-matching.fasl")
(#xfa9f871b0fafe08cf517547d6c53dacb . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x2260896dfd7a74a934914daa1e3216a6 . "./code.cg2js/cg-picture-widget.fasl")
(#xc410a5fc94e432efeff07fe69ce092a5 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x1806c13d30c7e9646386d5916555202a . "./code.cg2js/cg-pixmap-printing.fasl")
(#x64afeff87b62a17df07a6ead74a7c73d . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xb5179abc36a9210d6da3b85717bc4ab5 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x3d770d6cf15360c9c67c494ac6044ac3 . "./code.cg2js/cg-pixmap.fasl")
(#x5bb00eabbcc0ce7208d7e906e6ec22ea . "./code.cg2js/cg-plot-widget.fasl")
(#xe73acb83591c06090cc0c89a242f52e9 . "./code.cg2js/cg-printing.fasl")
(#x74397719e537845e7c98ab1ffef52173 . "./code.cg2js/cg-progress-indicator.fasl")
(#x62bf29db20dff592838f31dfb1814b9f . "./code.cg2js/cg-project-window.fasl")
(#x5e28b01c821eef7dc31d9e411f06a9a2 . "./code.cg2js/cg-property.fasl")
(#x9147749446084178d8c2fa08ec633c64 . "./code.cg2js/cg-radio-button.fasl")
(#x3e22537f7c3756a9ce388c0143d76002 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xc5c44ff39f36ba9fe8ac585907944497 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xa281053e066484efc3aabac502e7f16d . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x8364310b8f6793fee85c83d96a929108 . "./code.cg2js/cg-rich-edit.fasl")
(#xc24f0616613680e474fd2850551b5d7a . "./code.cg2js/cg-sample-file-menu.fasl")
(#x19bfe9d104a30351a3e294616c611999 . "./code.cg2js/cg-scaling-stream.fasl")
(#xba2d7c4bda7c7eb83c9e613ca8bccab4 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xb46cbeb7a0276edfbba2102986872b27 . "./code.cg2js/cg-scroll-bar.fasl")
(#x490007197028a46c19b221fa5580362a . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x5c3f17801c0d99319030e50ac189aa63 . "./code.cg2js/cg-selected-object.fasl")
(#x96bd7c00f6807d22b3a0ff05b7fb8b47 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xaa7f5fd85600f694a8dbbb8d97159ffe . "./code.cg2js/cg-split-bar.fasl")
(#xbc0f3c40321d6efa177ba6b96adcbbf7 . "./code.cg2js/cg-static-text.fasl")
(#xd9f79c352cadc890221733509955c543 . "./code.cg2js/cg-status-bar.fasl")
(#xd9bf1d8d32169814642a2babcfe71962 . "./code.cg2js/cg-string-dialog.fasl")
(#xdb3bf658314f5faa314b8c52e269618d . "./code.cg2js/cg-tab-control.fasl")
(#x69729dc02d0b00582517ee59140b3d10 . "./code.cg2js/cg-template-string.fasl")
(#xbbc3ce3c7228c0927458cede7385ac61 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x29ddfe5a78191106e3b3368969589ffe . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xdfbd2d8edf6efe4128539b3cbb5d6438 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x1ef066732047491f29fbaf181327f31a . "./code.cg2js/cg-text-or-combo.fasl")
(#x7d3f9b63d00aa78377e9008ba262d2b7 . "./code.cg2js/cg-text-widget.fasl")
(#x6b074106345a392135a6ac427bdbd724 . "./code.cg2js/cg-timer.fasl")
(#x53a1bf460f874d2d3c02e82d7748c715 . "./code.cg2js/cg-toggling-widget.fasl")
(#x1fd4e2f301aadcf1b384881eb0646c95 . "./code.cg2js/cg-toolbar.fasl")
(#x5535fd0236453965759fe751524bcda8 . "./code.cg2js/cg-tooltip.fasl")
(#x87dc347ed3eac27186add91697852101 . "./code.cg2js/cg-touch.fasl")
(#xf77a8c9cf136974f5afd4eb448920433 . "./code.cg2js/cg-trackbar.fasl")
(#x0141120150c54b6ca09cef97f36dac77 . "./code.cg2js/cg-tray.fasl")
(#x133cbb9f12ac3727f561b9869d5d7812 . "./code.cg2js/cg-up-down-control.fasl")
(#x277edd9417047feab2b61ea995529de6 . "./code.cg2js/cg-utility-dialog.fasl")
(#xb863a64b69516f2b517ec35b29127bfd . "./code.cg2js/cg-web-browser-dde.fasl")
(#x137f7bad4269c478eef8f4a97009c7bb . "./code.cg2js/cg-web-browser.fasl")
(#xc91f98e1a8904c043480db20cbbb3cac . "./code.cg2js/cg-wrap-string.fasl")
(#xa41f3256d5ca9c925197fb0cbfcd33a6 . "./code.cg2js/cg-yes-no-list.fasl")
(#xefe5fd10b41d531991054793ad9b5bce . "./code.cg2js/cg-yes-no-string.fasl")
(#xf3c9de78f3edb0f355de27222092640b . "./code.cg2js/cg.fasl")
(#x7039b06fc5394c56cb60733c64bccede . "./code.idejs/ide-acache.fasl")
(#xf3288b606ccda074388076e1eb76bcca . "./code.idejs/ide-apropos.fasl")
(#xcd12e294e4903104acdd6ec002c4a6ac . "./code.idejs/ide-base.fasl")
(#x13102cab74abd8333ae73bbd2e350486 . "./code.idejs/ide-class-browser.fasl")
(#xbce7de2bffa242e0b20a60a674f28ad4 . "./code.idejs/ide-class-grapher.fasl")
(#x931dd385dcdf005b57cd6f3b1538000c . "./code.idejs/ide-clipboard.fasl")
(#xf7ddbc16cb2d1e7a7798c49b29af8da2 . "./code.idejs/ide-dev.fasl")
(#xac0e42e41a417dbf4e285288b0821075 . "./code.idejs/ide-editor.fasl")
(#xcd195290fccff1635af4cdde2ffb3c87 . "./code.idejs/ide-find-definitions.fasl")
(#x5d0377106f6609f8d105237c86a54860 . "./code.idejs/ide-find-in-files.fasl")
(#x426f523f9d0d035029ae978160f38adf . "./code.idejs/ide-form.fasl")
(#x4f07fe3ebbc1d2c11a242cd4fd0c146c . "./code.idejs/ide-grapher.fasl")
(#xbec9ac4b3c184bfd8b0d6f57ebbab99f . "./code.idejs/ide-inspector.fasl")
(#xac3b1b9a60cf4c6c2936f0bdf4092a18 . "./code.idejs/ide-intro.fasl")
(#x07ea134a94e2afdd64960f3bf67c4a2b . "./code.idejs/ide-keyshort.fasl")
(#x2f33440c9fa78490c8968a1676eddfc1 . "./code.idejs/ide-navigator.fasl")
(#xc964e912c4369b2c8381773985e55d1f . "./code.idejs/ide-package-list.fasl")
(#x66962f543e1ee425b0e537fd20121fc1 . "./code.idejs/ide-patch-dialog.fasl")
(#xb6ae1804c7c86cd9ea0411d967e58fe1 . "./code.idejs/ide-process-browser.fasl")
(#x6c97d022a4848c8051d31521912e6261 . "./code.idejs/ide-project.fasl")
(#x30c9dc87777b2c28aec7ea82bfafc5d7 . "./code.idejs/ide-trace.fasl")
(#x4b72e5ac40c5f5e7f0597fe5fc0d0196 . "./code.idejs/ide-window-list.fasl")
(#x6f70b2d50134778b7554194e42c623a0 . "./code.idejs/ide.fasl")
(#x7f480facc219636edfe865d5e108e9cb . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
